using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Drawing.Design;
using System.Text;
using System.Windows.Forms;
using Telerik.WinControls;
using Telerik.WinControls.Design;
using Telerik.WinControls.Layouts;
using Telerik.WinControls.Primitives;
using Telerik.WinControls.UI;

namespace WindowsFormsApplication3
{
    public class CollapsiblePaneElement : StackLayoutElement
    {
        #region Rad Properties

        public static RadProperty ExpandedProperty = RadProperty.Register("Expanded", typeof(bool),
          typeof(CollapsiblePaneElement), new RadElementPropertyMetadata(true, ElementPropertyOptions.AffectsLayout));

        #endregion

        #region Fields

        private LightVisualElement caption;
        private StackLayoutElement body;
        private RadItemOwnerCollection items;
        private Padding savedPadding;

        #endregion

        #region Initialization

        static CollapsiblePaneElement()
        {
            ItemStateManagerFactoryRegistry.AddStateManagerFactory(new CollapsiblePaneElementStateManagerFactory(), typeof(CollapsiblePaneElement));
        }

        protected override void InitializeFields()
        {
            base.InitializeFields();

            this.DrawBorder = true;
            this.DrawFill = true;
            this.StretchHorizontally = true;
            this.StretchVertically = false;

            this.items = new RadItemOwnerCollection();
            this.items.ItemTypes = new Type[] { typeof(RadButtonElement) };

            this.Orientation = System.Windows.Forms.Orientation.Vertical;
        }

        protected override void CreateChildElements()
        {
            caption = new LightVisualElement();
            caption.DrawBorder = true;
            caption.DrawFill = true;
            caption.TextImageRelation = System.Windows.Forms.TextImageRelation.ImageBeforeText;
            caption.ImageAlignment = ContentAlignment.MiddleLeft;
            caption.StretchVertically = false;
            caption.Class = "Caption";
            caption.NotifyParentOnMouseInput = true;
            this.Children.Add(caption);

            body = new StackLayoutElement();
            body.Orientation = System.Windows.Forms.Orientation.Vertical;
            body.StretchHorizontally = true;
            body.StretchVertically = true;
            body.DrawFill = true;
            body.Class = "Body";
            this.Children.Add(body);
            this.Items.Owner = body;
        }
        
        #endregion

        #region Properties

        public LightVisualElement CaptionElement
        {
            get
            {
                return this.caption;
            }
        }

        public StackLayoutElement BodyElement
        {
            get
            {
                return this.body;
            }
        }

        [RadPropertyDefaultValue("Expanded", typeof(CollapsiblePaneElement))]        
        public bool Expanded
        {
            get
            {
                return (bool)this.GetValue(ExpandedProperty);
            }
            set
            {
                this.SetValue(ExpandedProperty, value);
            }
        }
      
        [DefaultValue("")]
        public string Caption
        {
            get
            {
                return this.caption.Text;
            }
            set
            {
                this.caption.Text = value;
            }
        }
      
        [DefaultValue(null)]
        public Image CaptionImage
        {
            get
            {
                return this.caption.Image;
            }
            set
            {
                this.caption.Image = value;
            }
        }

        [DesignerSerializationVisibility(DesignerSerializationVisibility.Content)]
        public RadItemOwnerCollection Items
        {
            get
            {
                return this.items;
            }
        }

        #endregion

        #region Event handlers

        protected override void OnBubbleEvent(RadElement sender, RoutedEventArgs args)
        {
            base.OnBubbleEvent(sender, args);

            LightVisualElement element = sender as LightVisualElement;
            if (element != null && element.ThemeRole == "Caption" && args.RoutedEvent == RadItem.MouseDownEvent)
            {
                Expanded = !Expanded;
            }
        }

        protected override void OnPropertyChanged(RadPropertyChangedEventArgs e)
        {
            base.OnPropertyChanged(e);

            if (e.Property == ExpandedProperty)
            {
                if ((bool)e.NewValue)
                {
                    AnimatedPropertySetting setting = new AnimatedPropertySetting(RadElement.PaddingProperty,
                        this.BodyElement.Padding, savedPadding, 8, 40);
                    setting.ApplyEasingType = RadEasingType.OutQuad;
                    setting.ApplyValue(this.BodyElement);

                    AnimatedPropertySetting setting2 = new AnimatedPropertySetting(LightVisualElement.DrawFillProperty,
                        this.BodyElement.DrawFill, false, 8, 40);
                    setting2.ApplyValue(this);

                    AnimatedPropertySetting setting3 = new AnimatedPropertySetting(LightVisualElement.OpacityProperty,
                        this.BodyElement.Opacity, 1d, 8, 40);
                    setting3.ApplyEasingType = RadEasingType.OutQuad;
                    setting3.ApplyValue(this.BodyElement);
                }
                else
                {
                    savedPadding = this.BodyElement.Padding;
                    Padding newPadding = savedPadding;
                    newPadding.Top -= this.BodyElement.Size.Height;

                    AnimatedPropertySetting setting = new AnimatedPropertySetting(RadElement.PaddingProperty,
                        this.BodyElement.Padding, newPadding, 8, 40);
                    setting.ApplyEasingType = RadEasingType.InQuad;
                    setting.ApplyValue(this.BodyElement);

                    AnimatedPropertySetting setting2 = new AnimatedPropertySetting(LightVisualElement.DrawFillProperty,
                        this.BodyElement.DrawFill, true, 8, 40);
                    setting2.ApplyDelay = 150;
                    setting2.ApplyValue(this);

                    AnimatedPropertySetting setting3 = new AnimatedPropertySetting(LightVisualElement.OpacityProperty,
                        this.BodyElement.Opacity, 0d, 8, 40);
                    setting3.ApplyEasingType = RadEasingType.OutQuad;
                    setting3.ApplyValue(this.BodyElement);
                }   
            }
        }

        #endregion
    }
}